@@ -1,5 +1,5 @@
 package Sim::OPT::Interlinear;
-# IMPORTANT: TO USE THE PROGRAM AS A SCRIPT, THE ABOVE "package Sim::OPT::Interlinear;" line must be deleted.
+# NOTE: TO USE THE PROGRAM AS A SCRIPT, THE LINE ABOVE SHOULD BE DELETED.
 
 # INTERLINEAR
 # Author: Gian Luca Brunetti, Politecnico di Milano. (gianluca.brunetti@polimi.it)
@@ -29,12 +29,12 @@ use Sim::OPT::Report;
 use Sim::OPT::Descend;
 use Sim::OPT::Takechance;
 use Sim::OPT::Parcoord3d;
-# IMPORTANT: TO USE THE PROGRAM AS A SCRIPT, THE ABOVE "use Sim::OPT..." lines should be deleted, unless SIM::OPT is installed.
+# NOTE: TO USE THE PROGRAM AS A SCRIPT, THE ABOVE "use Sim::OPT..." lines should be deleted.
 
 our @ISA = qw( Exporter );
 our @EXPORT = qw( interlinear, interstart prepfactlev tellstepsize );
 
-$VERSION = '0.139';
+$VERSION = '0.143';
 $ABSTRACT = 'Interlinear is a program for building metamodels from incomplete, multivariate, discrete dataseries on the basis of nearest-neighbouring gradients weighted by distance.';
 
 #######################################################################
@@ -1040,10 +1040,9 @@ sub wei
   {
     say $tee "Entering gradients' \%bank.";
     my ( $bank_ref, $nears_ref ) = fillbank( \@arr__, $minimumcertain, $minreq_forgrad, $maxdist, $condweight, \%factlevels, $nfilter, \@arra, $first0, $last0, \%nears );
-    %bank = %{ $bank_ref };
+    %bank = %{ $bank_ref }; say $tee "\%bank: " . dump( %bank );
     %nears = %{ $nears_ref };
     %bank = %{ clean( \%bank, $nfilter, "principal", \@recedes ) }; say $tee "BANK DONE."; #say $tee "CLEANED \%bank: " . dump( %bank ) ;
-    my %bank =  %{ $bank_ref }; #_#say $tee "CLEANED \%bank: " . dump( %bank );
   }
 
 ###--###
@@ -1053,22 +1052,25 @@ sub wei
     my %bank = %{ $bank_ref };
     my %weldbank = %{ $weldbank_ref };
     my @parsws = @{ $parsws_ref };
-    say $tee "Welding " . dump( @parsws );
+    say $tee "Mixing for welding " . dump( @parsws );
     foreach my $trio ( keys %weldbank )
     {
-      my @splits = split( "-", $trio );
-      my $num = $split[0];
-      if ( $num ~~ @parsws )
+      unless ( $trio eq "" )
       {
-        push ( @{ $bank{$trio}{orderedtrio} }, @{ $weldbank{$trio}{orderedtrio} } );
-        push ( @{ $bank{$trio}{grad} }, @{ $weldbank{$trio}{grad} } );
-        push ( @{ $bank{$trio}{ordists} }, @{ $weldbank{$trio}{ordists} } );
-        push ( @{ $bank{$trio}{strengths} }, @{ $weldbank{$trio}{strengths} } );
-        push ( @{ $bank{$trio}{orvals} }, @{ $weldbank{$trio}{orvals} } );
-        push ( @{ $bank{$trio}{origins} }, @{ $weldbank{$trio}{origins} } );
-        push ( @{ $bank{$trio}{orstrings} }, @{ $weldbank{$trio}{orstrings} } );
-        push ( @{ $bank{$trio}{orstring} }, @{ $weldbank{$trio}{orstring} } );
-        say $tee "weld $num.";
+        my @splits = split( "-", $trio ); say $tee "\@splits " . dump( @splits );
+        my $num = $splits[0];
+        if ( $num ~~ @parsws )
+        {
+          push ( @{ $bank{$trio}{orderedtrio} }, @{ $weldbank{$trio}{orderedtrio} } );
+          push ( @{ $bank{$trio}{grad} }, @{ $weldbank{$trio}{grad} } );
+          push ( @{ $bank{$trio}{ordists} }, @{ $weldbank{$trio}{ordists} } );
+          push ( @{ $bank{$trio}{strengths} }, @{ $weldbank{$trio}{strengths} } );
+          push ( @{ $bank{$trio}{orvals} }, @{ $weldbank{$trio}{orvals} } );
+          push ( @{ $bank{$trio}{origins} }, @{ $weldbank{$trio}{origins} } );
+          push ( @{ $bank{$trio}{orstrings} }, @{ $weldbank{$trio}{orstrings} } );
+          push ( @{ $bank{$trio}{orstring} }, @{ $weldbank{$trio}{orstring} } );
+          say $tee "weld $num.";
+        }
       }
     }
     return( \%bank );
@@ -1079,7 +1081,7 @@ sub wei
     my $co = 0;
     foreach my $weldref ( @weldaarrs )
     {
-      my @parsws = @{ $parswelds[$co] }; #say $tee "WELDING: " . dump( @parsws );
+      my @parsws = @{ $parswelds[$co] }; say $tee "\@parsws: " . dump( @parsws );
       my @weldarr = @{ $weldref }; #_#say $tee "THIS \@weldarr: " . dump( @weldarr ) . " DONE THIS \@weldarr.";
       my @weldarr__;
       if ( ( $limit_checkdistgrades ne "" ) or ( $limit_checkdistpoints ne "" ) )
@@ -1116,15 +1118,17 @@ sub wei
       {
         @weldarrb = @weldarr__;
       }
-
+      say $tee "FIRST0 $first0,  LAST0 $last0 ";
+      #say $tee " \@weldarr__: " . dump( \@weldarr__ ) ;
       my ( $weldbank_ref, $weldnears_ref ) = fillbank( \@weldarr__, $minimumcertain, $minreq_forgrad, $maxdist, $condweight, \%factlevels, $nfilter, \@weldarra, $first0, $last0, \%nears );
-      %weldbank = %{ $weldbank_ref };
+      %weldbank = %{ $weldbank_ref }; say $tee " \%weldbank: " . dump( \%weldbank ) ;
       %weldnears = %{ $weldnears_ref };
 
-      %weldbank = %{ clean( \%weldbank, $nfilter, "weld" ) }; #_#say $tee "CLEANED \%weldbank: " . dump( \%weldbank ) ;
-      #my %weldbank =  %{ $weldbank_ref }; say $tee "CLEANED \%weldbank: " . dump( %weldbank );
+      %weldbank = %{ clean( \%weldbank, $nfilter, "weld" ) }; #say $tee "CLEANED \%weldbank: " . dump( \%weldbank ) ;
 
-      %bank = %{ mixbank( \%bank, \%weldbank, \@parsws ) }; #_#say $tee "COMPLETED \%bank." . dump( %bank );
+      %bank = %{ mixbank( \%bank, \%weldbank, \@parsws ) };
+      say $tee "COMPLETED \%bank:" . dump( \%bank ) . ": COMPLETED \%bank.";
+      say $tee "KEYS \%bank: " . ( keys %bank );
 
       $co++;
     }
@@ -1918,51 +1922,7 @@ sub interlinear
   else
   {
 
-    $maxloops= 1000;
-    $sourcefile = "/home/luca/fexpexp/startersprep.csv";
 
-    @weldsprepared = ( "/home/luca/fexpexp/inmissionsprep.csv" );
-    @parswelds = ( [ 1 ] );
-
-    @recedes = ( 6 );
-
-    $newfile = $sourcefile . "_meta.csv";
-    $report = $newfile . "_report.txt";
-    @mode = ( "wei" ); # #"wei" is weighted gradient linear interpolation of the nearest neighbours.
-    #my @mode = ( "near" ); # "nea" means "nearest neighbour"
-    #@mode = ( ""mix" ); # "mix" means sequentially mixed in each loop.
-    #my @mode = ( "wei", "near", "near", "purelin" ); # a sequence
-    @weights = (  ); #my @weights = ( 0.7, 0.3 ); # THE FIRST IS THE WEIGHT FOR linear interpolation, THE SECOND FOR nearest neighbour.
-    # THEY ARE FRACTIONS OF 1 AND MUST GIVE 1 IN TOTAL. IF THE VALUES ARE UNSPECIFIED (IE. IF THE ARRAY IS EMPTY), THE VALUES ARE UNWEIGHTED.
-    $linearprecedence = "no"; # PRECEDENCE TO LINEAR INTERPOLATES. IF "yes", THE VALUES DERIVED FROM LINEAR INTERPOLATION, WHERE PRESENT, WILL SUPERSEDE THE VALUES DERIVED FROM THE NEAREST NEIGHBUR STRATEGY. IF "no", THE OPPOSITE. IT CAN BE ACTIVE ONLY IF LOGARITHMIC IS OFF. IT WORKS WITH "LINEAR" AND "EXPONENTIAL"
-
-    $relaxmethod = "logarithmic"; #It is relative to the "linnear" method. Options: "logarithmic", "linear" or "exponential". THE FARTHER NEIGHBOURS ARE WEIGHT LESS THAN THE NEAREST ONES: INDEED, LOGARITHMICALLY, LINEARLY OR EXPONENTIALLY. THE LOGARITHM BASE IS $relaxlimit OR $nearrelaxlimit, DEPENDING FROM THE CONTEXT. THE LINEAR MULTIPLICATOR OR THE EXPONENT IS DEFINED BY $overwerightnearest.
-    $relaxlimit = 1; #THIS IS THE CEILING FOR THE RELAXATION OF THE RELATIONS OF PURE LINEAR INTERPOLATION. For greatest precision: 0, or a negative number near 0. IF > = 0, THE PROGRAM INTERPOLATES ALSO NEAREST NEIGHBOURS. THE HIGHER THE NUMBER, THE FARTHER THE NEIGHBOURS.
-    $overweightnearest = 1; #A NUMBER. IT IS MADE NULL BY THE $logarithmicrelax "yes". THE HIGHER THE NUMBER, THE GREATER THE OVERWEIGHT GIVEN TO THE NEAREST. IN THIS MANNER, THE OVERWEIGHT IS NOT LOGARITHMIC, LIKE IT WHERE OTHERWISE, BUT LINEAR. THIS SLOWS DOWN THE COMPUTATIONS. UNLESS THE OVERWEIGHT IS 1, WHICH MAKES THE OVERWEIGHTING NULL.
-    $nearrelaxlimit = 0; #THIS IS THE CEILING FOR THE RELAXATION OF THE RELATIONS OF THE NEAREST NEIGHBOUR STRATEGY. For greatest precision: 0, or a negative number near 0. IF > = 0, THE PROGRAM INCREASES THE DISTANCE OF THE NEAREST NEIGHBOURS INCLUDED. THE HIGHER THE NUMBER, THE FARTHER THE NEIGHBOURS. ONE IS NOT LIKELY TO WANT TO USE THIS OPTION.
-    $nearconcurrencies = 1; #Requested minimum number of concurrencies for the nearest neighbour method. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
-    $parconcurrencies = 1; #Requested minimum number of concurrencies for the linear interpolations for each parameter of each instance. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
-    $instconcurrencies = 1; #Requested minimum number of concurrencies for the linear interpolations for each instance. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
-    # my %factlevels = ( pairs => { 1=>3, 2=>3, 3=>3, 4=>3, 5=>3 } ); #The keys are the factor numbers and the values are the number of levels in the data series. OBSOLETE.
-    $minreq_forgrad = [1, 1, 1 ]; #THIS VALUES SPECIFY THE NUMBER OF PARAMETER DIFFERENCES (INTEGER NUMBERS) TELLING HOW WELL-ROOTED IN SIMULATED REALITY (I.E, NEAR FROM IT) A POINT MUST BE TO SELECT IT FOR CALCULATING THE GRADIENTS FOR THE METAMODEL. THEY ARE INTEGERS STARTING FROM 1 OR GREATER.
-    # THE FIRST VALUE IS RELATIVE TO THE FACTORS AND TELLS HOW RELAXED A SEARCH IS.
-    # THE SECOND VALUE IS RELATIVE TO THE LEVELS. ONE MAY WANT TO KEEP IT TO 1: THE GRADIENTS ARE CALCULATED USING ONLY ADJACENT INSTANCES.
-    # ONE MAY WANT TO KEEP IT TO 1: THE GRADIENTS ARE CALCULATED USING ONLY ADJACENT INSTANCES.
-    # THE THIRD VALUE HOW DIFFERENT (FAR, IN TERMS OF PARAMETER DIFFERENCES) MAY AN INSTANCE BE TO BE CALCULATED THROUGH THE GRADIENT IN QUESTION.
-    # ONE MAY WANT TO SET TO THE NUMBER OF PARAMETERS.
-    # A LARGE NUMBER, WEAK ENTRY BARRIER. NEVER LESS THAN 1.
-
-    $minreq_forinclusion = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT (META-POINT) MUST BE FOR INCLUDING IT IN THE SET OF POINTS USED FOR THE METAMODEL.If 0, no entry barrier.
-    $minreq_forcalc = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT MUST BE FOR INCLUDING IT IN THE CALCULATIONS FOR CREATING NEW META-POINTS.  A VALUE BETWEEN 1 (JUST SIMULATED POINT) AND 0. If 0, no entry barrier.
-    $minreq_formerge = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT MUST BE FOR MERGING IT IN THE CALCULATIONS FOR MERGING IT IN THE METAMODEL. A VALUE BETWEEN 1 (JUST SIMULATED POINT) AND 0 (SIMULATED POINTS AND "META"POINTS WITH THE SAME RIGHT ) MUST BE SPECIFIED. If 0, no entry barrier.
-    $minimumcertain = 0; # WHAT IS THE MINIMUM LEVEL OF STRENGTH (LEVEL OF RELIABILITY) REQUIRED TO USE A DATUM TO BUILD UPON IT. IT DEPENDS ON THE DISTANCE FROM THE ORIGINS OF THE DATUM. THE LONGER THE DISTANCE, THE SMALLER THE STRENGTH (WHICH IS INDEED INVERSELY PROPORTIONAL). A STENGTH VALUE OF 1 IS OF A SIMULATED DATUM, NOT OF A DERIVED DATUM. If 0, no entry barrier.
-    $minimumhold = 1; # WHAT IS THE MINIMUM LEVEL OF STRENGTH (LEVEL OF RELIABILITY) REQUIRED FOR NOT AVERAGING A DATUM WITH ANOTHER, DERIVED DATUM. USUALLY IT HAS TO BE KEPT EQUAL TO $minimimcertain.  If 1, ONLY THE MODEL DATA ARE NOT SUBSTITUTABLE IN THE METAMODEL.
-    $condweight = "yes"; # THIS CONDITIONS TELLS IF THE STRENGTH (LEVEL OF RELIABILITY) OF THE GRADIENTS HAS TO BE CUMULATIVELY TAKEN INTO ACCOUNT IN THE WEIGHTING CALCULATIONS.
-    $nfilter = "100"; # do not take into account the gradients which in the ranking of strengths are below a certain position. If unspecified: inactive.
-    $limit_checkdistgrades = ""; # LIMIT OF RELATIONS TAKEN INTO ACCOUNT IN CALCULATING THE NET OF GRADIENTS. IF NULL, NO BARRIER. 10000 IS A GOOD COMPROMISE BETWEEN SPEED AND RELIABILITY.
-    $limit_checkdistpoints = ""; # LIMIT OF RELATIONS TAKEN INTO ACCOUNT IN CALCULATING THE NET OF POINTS. IF NULL, NO BARRIER. 10000 IS A GOOD COMPROMISE BETWEEN SPEED AND RELIABILITY.
-    $fulldo = "yes"; # TO SEARCH FOR MAXIMUM PRECISION AT THE EXPENSES OF SPEED. "yes" MAKES THE GRADIENTS BE RECALCULATED AT EACH COMPUTATION CYCLE. "super" even more. "tight" NO.
-    $lvconversion = ""; # IF "EQUAL", A FRACTION OF LEVEL HAS THE VALUE OF 1, WHATEVER THE NUMBER OF A FRATION OF LEVEL IN A FACTOR. OTHERWISE, A NUMBER.
 
   }
 
@@ -1979,7 +1939,7 @@ sub interlinear
   if ( $blockelts_r ne "" ){ @blockelts = @{ $blockelts_r }; } #say $tee "CHECK5 \@blockelts: " . dump( @blockelts );
 
   my @mode = adjustmode( $maxloops, \@mode );
-  say "Opening $sourcefile";
+  say $tee "Opening $sourcefile";
   open( SOURCEFILE, "$sourcefile" ) or die;
   my @lines = <SOURCEFILE>; #say $tee "REALLY \@lines: " . dump( @lines );
   close SOURCEFILE;
